﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Web.Script.Serialization;


namespace prj_ReportDataSet
{
	public class RestClient
	{
		private readonly JavaScriptSerializer _serializer;

		/// <summary>
		/// Initializes a new instance of the <see cref="RestClient"/> class.
		/// </summary>
		public RestClient()
		{
			_serializer = new JavaScriptSerializer();
		}

		public AuthToken AuthToken { get; private set; }
		public Uri ArsUrl { get; set; }

		/// <summary>
		/// Logins to the ARS.
		/// </summary>
		/// <param name="user">The user name.</param>
		/// <param name="password">The user password.</param>
		/// <returns>The instance of the <see cref="AuthToken"/> class.</returns>
		public void Login(string user, string password)
		{
			var data = _serializer.Serialize(new LoginRequest {User = user, Password = password});
			AuthToken = Post<AuthToken>(Constants.LoginUri, data);
		}

		
        /// <summary>
        /// Get the report Content.
        /// </summary>
        /// <param name="reportId">The report id.</param>
        public string GetReportContent(string reportId)
        {
            var uri = string.Format(Constants.GetReportContent, reportId);
            using (var customWebClient = CreateWebClient())
            {
                var responseData = customWebClient.DownloadString(uri);
                if (customWebClient.HttpStatusCode == HttpStatusCode.OK)
                    return responseData;
                else
                    return null;
            }
               
 
        }
       
		#region Helpers

		private void Post(string uri, string data)
		{
			using (WebClient webClient = CreateWebClient())
			{
				webClient.UploadString(uri, HttpMethod.Post.Method, data);
			}
		}

		private T Post<T>(string uri, string data)
		{
			using (WebClient webClient = CreateWebClient())
			{
				return _serializer.Deserialize<T>(webClient.UploadString(uri, HttpMethod.Post.Method, data));
			}
		}

		private T Get<T>(string uri)
		{
			using (WebClient webClient = CreateWebClient())
			{
				return _serializer.Deserialize<T>(webClient.DownloadString(uri));
			}
		}

		private CustomWebClient CreateWebClient()
		{
			var webClient = new CustomWebClient { BaseAddress = ArsUrl.AbsoluteUri };
			webClient.Encoding = Encoding.UTF8;
			if (AuthToken != null && !string.IsNullOrWhiteSpace(AuthToken.Token))
				webClient.Headers.Add(Constants.AuthTokenHeader, AuthToken.Token);

			return webClient;
		}		

		#endregion
	}

	public class CustomWebClient : WebClient
	{
		public new WebHeaderCollection ResponseHeaders { get; private set; }
		public HttpStatusCode HttpStatusCode { get; private set; }
		
		protected override WebResponse GetWebResponse(WebRequest request)
		{
			var response = base.GetWebResponse(request);

			ResponseHeaders = response.Headers;

			var httpWebResponse = response as HttpWebResponse;
			if (httpWebResponse != null)
				HttpStatusCode = httpWebResponse.StatusCode;

			return response;
		}
	}
}
